
;*********************************************************************
;  Write LCD-Command Chip 1...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_control_1:
		out		LCD_DATA,TEMP_A

		cbi		LCD_CTRL,LCD_DI
        cbi		LCD_CTRL,LCD_CSA

		rcall	lcd_e_clock

        sbi		LCD_CTRL,LCD_DI
        sbi		LCD_CTRL,LCD_CSA

		ret

;*********************************************************************
;  Write LCD-Command Chip 2...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_control_2:
		out		LCD_DATA,TEMP_A

		cbi		LCD_CTRL,LCD_DI
        cbi		LCD_CTRL,LCD_CSB

		rcall	lcd_e_clock

        sbi		LCD_CTRL,LCD_DI
        sbi		LCD_CTRL,LCD_CSB

		ret

;*********************************************************************
;  Write LCD-Daten Chip 1...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_daten_1:
		out		LCD_DATA,TEMP_A

        cbi		LCD_CTRL,LCD_CSA

		rcall	lcd_e_clock

        sbi		LCD_CTRL,LCD_CSA

		ret

;*********************************************************************
;  Write LCD-Daten Chip 2...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_daten_2:
		out		LCD_DATA,TEMP_A

        cbi		LCD_CTRL,LCD_CSB

		rcall	lcd_e_clock

        sbi		LCD_CTRL,LCD_CSB

		ret

;*********************************************************************
;  LCD-E-Clock
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

lcd_e_clock:
		push	TEMP_A
        sbi     LCD_CTRL,LCD_E          ; E auf 1
		ldi		TEMP_A,(CYCLES_PER_US/3)+1

lcd_e_clock_loop_1:
		dec		TEMP_A
		brne	lcd_e_clock_loop_1

		cbi     LCD_CTRL,LCD_E          ; E auf 0
		ldi		TEMP_A,(CYCLES_PER_US*5)/3	; rund 5000ns

lcd_e_clock_loop_2:
		dec		TEMP_A
		brne	lcd_e_clock_loop_2

		pop		TEMP_A
		ret

;*********************************************************************
;  LCD-Init
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

lcd_init:
		rcall wait_5ms
			   	
		ldi TEMP_A, 0xC0			;Start Display Line = 0
		rcall lcd_control_1
		rcall lcd_control_2
		
		ldi		TEMP_A,0			; 1. Page

lcd_init_loop_2:
		push	TEMP_A	
		rcall	lcd_clear_line
		pop		TEMP_A
		inc		TEMP_A
		cpi		TEMP_A,8			; fertig?
		brne	lcd_init_loop_2

		ldi TEMP_A, 0x3F			;Display ein
		rcall lcd_control_1
		rcall lcd_control_2
				
		ret

;*********************************************************************
;  Lscht eine Page im Display-Ram, Nummer der Page in TEMP_A
;
;  Scratch-Reg: TEMP_A, TEMP_B
;*********************************************************************

lcd_clear_line:
		ori		TEMP_A,$B8
		rcall	lcd_control_1
		rcall	lcd_control_2
		ldi		TEMP_A,$40			; Column Adress (upper bit)
		rcall	lcd_control_1
		rcall	lcd_control_2
		ldi		TEMP_A,$00
		ldi		TEMP_B,$40

lcd_clear_loop:
		rcall	lcd_daten_1
		rcall	lcd_daten_2
        dec		TEMP_B
        brne	lcd_clear_loop

		ret

;*********************************************************************
;  Schreibt eine Zeile als Text 8x6 Font in Display-Ram 
;  In:  Nummer der Zeile in TEMP_A
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C, TEMP_D, ZH, ZL, YH, YL
;*********************************************************************

lcd_write_line:
		mov		TEMP_D,TEMP_A			; TEMP_A retten
		ori		TEMP_A,$B8
		rcall	lcd_control_1
		rcall	lcd_control_2
		ldi		TEMP_A,$44
		rcall	lcd_control_1
		ldi		TEMP_A,$40
		rcall	lcd_control_2
	
		load_p	Y,LCD_LINE				; auszugebender Text
		ldi		TEMP_B, CHAR_PER_LINE/2	; Anzahl Zeichen

lcd_write_line_loop_1:		
		ld		TEMP_A,Y+				; erstes ASCII-Zeichen holen
		subi	TEMP_A,$20				; Sonder-Codes weg
		ldi		TEMP_C,CHAR_WIDTH		; Zeichenbreite
		mul		TEMP_A,TEMP_C			; Offset zum Tabellenanfang
		load_p	Z,(2*char_tab_6)		; Start Tabelle 6x8 Font
		add		ZL,MUL_L
		adc		ZH,MUL_H

		ldi		TEMP_C,CHAR_WIDTH		; Zeichenbreite

lcd_write_line_char_loop_1:
		lpm		TEMP_A,Z+				; Bitmuster holen
		rcall	lcd_daten_1
        dec		TEMP_C
        brne	lcd_write_line_char_loop_1	; Zeichen fertig?

		dec		TEMP_B					; alle Zeichen?
		brne	lcd_write_line_loop_1	; nein, weiter

		ldi		TEMP_B, CHAR_PER_LINE/2	; Anzahl Zeichen

lcd_write_line_loop_2:		
		ld		TEMP_A,Y+				; erstes ASCII-Zeichen holen
		subi	TEMP_A,$20				; Sonder-Codes weg
		ldi		TEMP_C,CHAR_WIDTH		; Zeichenbreite
		mul		TEMP_A,TEMP_C			; Offset zum Tabellenanfang
		load_p	Z,(2*char_tab_6)		; Start Tabelle 6x8 Font
		add		ZL,MUL_L
		adc		ZH,MUL_H

		ldi		TEMP_C,CHAR_WIDTH		; Zeichenbreite

lcd_write_line_char_loop_2:
		lpm		TEMP_A,Z+				; Bitmuster holen
		rcall	lcd_daten_2
        dec		TEMP_C
        brne	lcd_write_line_char_loop_2	; Zeichen fertig?

		dec		TEMP_B					; alle Zeichen?
		brne	lcd_write_line_loop_2	; nein, weiter

		ret

;*********************************************************************
;  Schreibt eine Zeile als Text 16x12 Font in Display-Ram, Nummer der Zeile in TEMP_A
;  In:  Nummer der Zeile in TEMP_A
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C, TEMP_D, ZH, ZL, YH, YL
;*********************************************************************

lcd_write_line_big:
		push	TEMP_A					; Page-Nummer (Zeile) retten
		ldi		TEMP_D,2				; 2 Durchlufe fr 2 Zeilen Hhe des Fonts

lcd_write_line_big_loop_1_1:

		push	TEMP_A					; Page-Nummer (Zeile) retten
		ori		TEMP_A,$B8
		rcall	lcd_control_1			; Set Page
		ldi		TEMP_A, 0x44			; Column Adresse
		rcall	lcd_control_1
		
		load_p	Y,LCD_LINE				; auszugebender Text
		ldi		TEMP_B,CHAR_PER_LINE_BIG/2	; Zhler Anzahl Zeichen

lcd_write_line_big_loop_1_2:		
		ld		TEMP_A,Y+				; erstes ASCII-Zeichen holen
		subi	TEMP_A,$20				; Sonder-Codes weg
		ldi		TEMP_C,CHAR_WIDTH_BIG*2	; Zeichenbreite * 2 (Anzahl Bytes fr ein Zeichen)
		mul		TEMP_A,TEMP_C			; Offset zum Tabellenanfang
		load_p	Z,(2*char_tab_big)-1	; Start Tabelle mit Korrektur fr Offset in TEMP_D
		add		ZL,TEMP_D				; TEMP_D Durchlaufzhler, 2 - 1
		clr		TEMP_C
		adc		ZH,TEMP_C
		add		ZL,MUL_L				; Zeichenoffset zum Tabellenstart dazu
		adc		ZH,MUL_H

		ldi		TEMP_C,CHAR_WIDTH_BIG	; Zhler Zeichenbreite

lcd_write_line_big_char_loop_1_3:
		lpm		TEMP_A,Z+				; Bitmuster holen
		adiw	Z,1						; Muster fr unten berlesen
		rcall	lcd_daten_1
        dec		TEMP_C
        brne	lcd_write_line_big_char_loop_1_3		; Zeichen fertig?

		dec		TEMP_B					; alle Zeichen?
		brne	lcd_write_line_big_loop_1_2	; nein, weiter

		pop		TEMP_A					; Page-Nummer (Zeile)
		inc		TEMP_A					; nchste Zeile
		dec		TEMP_D					; nchter Durchlauf
		brne	lcd_write_line_big_loop_1_1

		pop		TEMP_A
		ldi		TEMP_D,2				; 2 Durchlufe fr 2 Zeilen Hhe des Fonts

lcd_write_line_big_loop_2_1:

		push	TEMP_A					; Page-Nummer (Zeile) retten
		ori		TEMP_A,$B8
		rcall	lcd_control_2			; Set Page
		ldi		TEMP_A, 0x40			; Column Adresse
		rcall	lcd_control_2
		
		load_p	Y,LCD_LINE+CHAR_PER_LINE_BIG/2	; auszugebender Text
		ldi		TEMP_B,CHAR_PER_LINE_BIG/2	; Zhler Anzahl Zeichen

lcd_write_line_big_loop_2_2:		
		ld		TEMP_A,Y+				; erstes ASCII-Zeichen holen
		subi	TEMP_A,$20				; Sonder-Codes weg
		ldi		TEMP_C,CHAR_WIDTH_BIG*2	; Zeichenbreite * 2 (Anzahl Bytes fr ein Zeichen)
		mul		TEMP_A,TEMP_C			; Offset zum Tabellenanfang
		load_p	Z,(2*char_tab_big)-1	; Start Tabelle mit Korrektur fr Offset in TEMP_D
		add		ZL,TEMP_D				; TEMP_D Durchlaufzhler, 2 - 1
		clr		TEMP_C
		adc		ZH,TEMP_C
		add		ZL,MUL_L				; Zeichenoffset zum Tabellenstart dazu
		adc		ZH,MUL_H

		ldi		TEMP_C,CHAR_WIDTH_BIG	; Zhler Zeichenbreite

lcd_write_line_big_char_loop_2_3:
		lpm		TEMP_A,Z+				; Bitmuster holen
		adiw	Z,1						; Muster fr unten berlesen
		rcall	lcd_daten_2
        dec		TEMP_C
        brne	lcd_write_line_big_char_loop_2_3		; Zeichen fertig?

		dec		TEMP_B					; alle Zeichen?
		brne	lcd_write_line_big_loop_2_2	; nein, weiter

		pop		TEMP_A					; Page-Nummer (Zeile)
		inc		TEMP_A					; nchste Zeile
		dec		TEMP_D					; nchter Durchlauf
		brne	lcd_write_line_big_loop_2_1

		ret

